#!/usr/bin/env bash
set -eu
set -o vi
shopt -s extglob lastpipe

declare -A TERM_STARTUP_OPTIONS=(
	[ssh]="SSH: connect to a remote host"
	[tmux]="Tmux: connect to local terminal multiplexer"
)

. ~/.shell/lib/shared.sh
. ~/.shell/lib/term.bash
. ~/.profile

declare -r CODE_HIGHLIGHT=$(tput -S <<<"setab 15"$'\n'"setaf 16")
declare -r CODE_RESET=$(tput sgr0)
declare -a OPTIONS=()
declare -i FOCUS=0
declare USE_TMUX=true

if [[ -v TMUX ]] || ! has tmux || ! tmux has-session; then
	USE_TMUX=false
	unset TERM_STARTUP_OPTIONS[tmux]
fi
if ! has ssh; then
	unset TERM_STARTUP_OPTIONS[ssh]
fi

for opt in "${!TERM_STARTUP_OPTIONS[@]}"; do
	OPTIONS+=( $opt="${TERM_STARTUP_OPTIONS[$opt]}" )
done

# Hard-code the following two options:
OPTIONS+=(
	shell="Shell: start a local shell ($SHELL)"
	manual="Enter command manually"
)


_clear() { clear; echo; }

interact() {
	local STATE=ssh
	trap 'echo; read -N1 -p "Press any key to exit"' EXIT
	while _clear && do_${STATE//-/_}; do
		has do_${STATE//-/_} || die "Unknown state: $STATE"
	done
	die "Command for the $STATE state failed: $_"
}

do_ssh() {
	local host args

	if [[ ! -v SSH_HOSTS ]]; then
		local IFS=$'\n',
		cut -f1 -d ' ' <~/.ssh/known_hosts |
			read -d '' -a SSH_HOSTS || true
	fi

	read_completions \
		"Connect to SSH host [empty for other options]" \
		host args -- "${SSH_HOSTS[@]}"

	[[ -z $host ]] && STATE=options && return
	exec ssh $host $args
}

do_tmux() {
	exec tmux attach
}

do_shell() {
	exec -l $SHELL
}

do_manual() {
	read -e -p '> '
	eval $REPLY
	exit
}

do_options() {
	local SELECTION
	while [[ ! -v SELECTION ]]; do
		display_options
		process_keys
	done
	local opt=${OPTIONS[$SELECTION]}
	STATE=${opt%%=*}
}

display_options() {
	_clear
	echo "Options:${1+ $1}"
	for i in "${!OPTIONS[@]}"; do
		local prefix=" " suffix=
		local opt=${OPTIONS[$i]}
		local name=${opt%%=*} desc=${opt#*=}
		[[ $i -eq ${1-$FOCUS} ]] && prefix="$CODE_HIGHLIGHT>" suffix=$CODE_RESET
		local rhs=$(tput hpa $((COLUMNS-${#name}-2)))"[$name]"
		printf "$prefix %2d: %s%s$suffix\n" $i "$desc" "$rhs"
	done
	echo
}

select_from() {
	local choice=$1
	if [[ $choice =~ ^[0-9]+$ && -v OPTIONS[choice] ]]; then
		SELECTION=$choice
		return 0
	fi

	local i=0
	declare -gA named_options
	if [[ ! -v named_options ]]; then
		named_options=()
		for opt in "${OPTIONS[@]}"; do
			named_options[${opt%%=*}]=$((i++))
		done
	fi

	if [[ ! -v named_options[$choice] ]]; then
		echo >&2 "Unknown choice: $choice"
		return 1
	fi

	SELECTION=${named_options[$choice]}
}

process_keys() {
	local key choice
	while read -s -N1 key; do
		case $key in
			$'\n') SELECTION=$FOCUS; break ;;
			$'\e') process_esc_seq ;;
			*) read -ei "$key" choice; select_from "$choice" && break ;;
		esac
	done
}

process_esc_seq() {
	local buff=
	local opt_min=0
	local opt_max=$((${#OPTIONS[*]}-1))
	while read -rs -N1 key; do
		buff+=$key
		case $buff in
			# arrow keys
			[[O]A) let 'FOCUS=(--FOCUS>=opt_min)?FOCUS:opt_min'; break ;;
			[[O]B) let 'FOCUS=(++FOCUS<=opt_max)?FOCUS:opt_max'; break ;;

			# any other ESC input sequences
			\[*([0-;])[A-Z~]) return ;;
		esac
	done
	display_options
}

has_client() {
	tmux list-clients -F '#{client_termname}' | while read termname; do
		if [[ $termname == $TERM ]]; then
			return 0
		fi
	done
	return 1
}

if $USE_TMUX && ! has_client; then
	# No other terms of this one's kind attached to Tmux, lets be the first
	exec tmux attach
fi

interact
