#!/bin/sh

[ -n "${SOURCED_SHELL_FUNCS-}" ] && return
SOURCED_SHELL_FUNCS=yes

. ~/.shell/lib/shared.sh

source_first()
{
	for sf in "$@"; do
		[ -f "$sf" ] && . "$sf" && return
	done
}

source_each()
{
	for sf in "$@"; do
		[ -f "$sf" ] && . "$sf"
	done
}

get_platform()
{
	case "$OSTYPE" in
		linux-gnu*) echo gnu ;;
		darwin*) echo darwin ;;
		*) echo $OSTYPE ;;
	esac
}

get_branch()
{
	git rev-parse --abbrev-ref HEAD
}

manage_path()
{
	local var="$1" val="$2"; shift 2
	eval local cur=\${$var-}
	local cur=`echo $cur | sed -E 's!(^|:)'"$val"'(:|$)!\1\2!;s!^:|:$!!;s!::!:!'`
	local pre="$cur" post=
	while [ $# -gt 0 ]; do
		case "$1" in
			begin*|before|infront)
				post="$cur"
				pre=
				;;
			check)
				test -d "$val" || return 0 ;;
		esac
		shift
	done
	eval $var="'${pre:+$pre:}$val${post:+:$post}'"
}

get_shell()
{
	if [ -n "$ZSH_NAME" ]; then echo zsh; return; fi
	if [ -n "$BASH" ]; then echo bash; return; fi
	echo sh
}

git_check()
{
	has let || return 0

	local v
	GIT_VERSION=0
	for v in `git --version|cut -f3 -d\ |tr '.' ' '|cut -f1-3 -d\ `; do
		let 'GIT_VERSION = (GIT_VERSION << 8) + (v & 0xff)'
	done

	if [ $GIT_VERSION -ge $((0x020900)) ]; then
		local numcpu
		if has lscpu; then
			numcpu=`lscpu -p|grep '^[^#]'|wc -l`
		elif [ -n "${NUMBER_OF_PROCESSORS:-}" ]; then
			numcpu="$NUMBER_OF_PROCESSORS"
		else
			numcpu=1
		fi
		GIT_JOBS="--jobs=$(((numcpu>1)?(numcpu-1):(numcpu)))"
		return
	fi

	GIT_JOBS=
}

update_worktree()
{
	local upstream=`git rev-parse --abbrev-ref --symbolic-full-name @{u}`
	if [ -z "$upstream" ]; then
		return 0
	fi
	local original=`git rev-parse HEAD`
	test $original = `git rev-parse $upstream` && return
	if git merge --ff --no-edit $upstream origin/master; then
		git submodule sync --recursive
		git submodule update ${GIT_JOBS} \
			--rebase \
			--init \
			--recursive \
			--depth=100
		foreach_submodule check_submodule
	else
		git reset $original
	fi
}

foreach_submodule()
{
	local name module SUBMODULE
	git config --list |
	sed -n 's/^submodule\.\(.\+\)\.url=.*$/\1/p' |
	while read name; do
		module=`git rev-parse --git-dir`/modules/$name || return 1

		# remove old leftover modules in config
		if ! [ -d "$module" ]; then
			git config --remove-section submodule."$name"
			continue
		fi

		# get worktree as SUBMODULE for eval
		SUBMODULE=`git config --file $module/config core.worktree`
		SUBMODULE=`cd $module/$SUBMODULE; pwd`

		(eval "$*")
	done
}

check_submodule()
{
	# exit early if status is fine
	git submodule status --quiet -- $SUBMODULE && return

	# If status returns 1 submodule removed from .gitmodules
	if [ $? -eq 1 ]; then
		test -e "$SUBMODULE" && rm -r "$SUBMODULE"
	fi

	git submodule deinit $SUBMODULE
}

do_update()
{
	local timestamp=${HOME}/.shell/.update
	local now=`get_utime` updated=`get_utime "$timestamp"`
	local BOLD_BLUE=$'\033[34;1m' DEFAULT=$'\033[m'
	local prompt="${BOLD_BLUE}Would you like to update dotfiles?${DEFAULT} "
	local shell=`get_shell`
	local unit ans

	if [ -n "${USE_PROMPT:-}" ]; then
		if [ $now -lt $((updated + 604800)) ]; then
			return
		fi

		case $shell in
			zsh) read "?$prompt" ans ;;
			*) read -p "$prompt" ans ;;
		esac
		case "$ans" in
			[Yy]|[Yy][Ee][Ss]) ;;
			[Nn]|[Nn][Oo]) touch "$timestamp"; return ;;
			*) return ;;
		esac
	fi

	finalise_timestamp() {
		code=$?
		unset -f finalise_timestamp
		test `get_shell` = bash && trap '' RETURN
		test -e "$1.old" && case $code in
			0) rm "$1.old" ;;
			*) mv "$1.old" "$1" ;;
		esac
	}
	trap "finalise_timestamp '$timestamp'" \
		`test $shell = bash && echo RETURN || echo EXIT`
	test -e "$timestamp" && mv "$timestamp" "$timestamp.old"
	touch "$timestamp"

	(cd ~
		set -eu
		git fetch origin &&
		git_check &&
		update_worktree
	) || return

	# re-source this file
	unset SOURCED_SHELL_FUNCS
	. ~/.shell/lib/shell-funcs.sh

	# ensure systemd units are enabled
	if type systemctl >/dev/null 2>&1; then
		systemctl --user daemon-reload
		for unit in ~/.config/systemd/user/*; do
			if [ -d "$unit" ]; then continue; fi
			echo "Enabling $unit"
			systemctl --user enable "${unit##*/}" 2>/dev/null
		done
	fi

	# enable cron scripts
	{ crontab -l || true; } |
		~/.shell/lib/cron-append.awk |
		crontab -
}

has resize || resize() {
	local IFS=';' flag signal
	case $(get_shell) in
		bash) flag=-a signal=RETURN ;;
		zsh) flag=-A signal=EXIT ;;
		*) echo >&2 "Cannot resize with this shell"; return 1 ;;
	esac
	stty -echo
	trap 'stty echo' $signal
	printf '\033[18t'
	read -d '[' -s _
	read -d 't' -s $flag params
	set -- "${params[@]}"
	[ $# -eq 3 -a $1 -eq 8 ] && shift
	[ $# -ne 2 ] && echo >&2 "Cannot resize" && return 1
	stty rows "$1" cols "$2" echo
	trap '' $signal
}
